/*  Chess-Ec      Copyright (C)   2001-2008 Marc Dechico

    ec_global.h : a common header file where global datas and functions
		are defined. 
		(working to change most of  them to static )

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    To contact the author:	Marc Dechico
			        Kerfring
			        29390 SCAER France
				tel: 06 37 98 22 54
				Mail: ec@chess-ec.org  
    
*/

#ifndef TYPES_H_   
#define TYPES_H
/* bug verify that define TYPES_H si correct */

/* AIX requires this to be the first thing in the file.  */
#if defined (_AIX) && !defined (__GNUC__)
 #pragma alloca
#endif
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <getopt.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <errno.h>


#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#if ENABLE_NLS
#include <libintl.h>
#define _(String) gettext (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#else
#define _(String) (String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#endif
#ifndef errno
extern int errno;
#endif

#ifdef	STDC_HEADERS
#include <stdlib.h>
#else	/* Not STDC_HEADERS */
extern void exit ();
extern char *malloc ();
#endif	/* STDC_HEADERS */

#ifdef	HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif

#ifdef	__GNUC__
#undef	alloca
#define	alloca(n)	__builtin_alloca (n)
#else	/* Not GCC.  */
#ifdef	HAVE_ALLOCA_H
#include <alloca.h>
#else	/* Not HAVE_ALLOCA_H.  */
#ifndef	_AIX
extern char *alloca ();
#endif	/* Not _AIX.  */
#endif	/* HAVE_ALLOCA_H.  */
#endif	/* GCC.  */

#ifdef HAVE_SYS_PARAM_H
/* To possibly get the definition of DEV_BSIZE. */
#include <sys/param.h>
#endif
#include "system.h"

#include	<ctype.h>
#include 	<signal.h>
#include        <sys/time.h>

/******************************************************************/
/* for including some  features                                   */
/******************************************************************/

//#define STHASH    /* one level implementation of hash           */
                    /* not compatible with UPLOAD_BRANCH          */

#define STHASH1     /* multi level implementation of hash         */
                    /* not compatible with UPLOAD_BRANCH          */

//#define UPLOAD_BRANCHE  /* upload the liste of move leadind to  */
                          /* use to build the best move           */ 
                        /* not compatible with STHASH AND STHASH1 */

/* LEA_PER_DEC not use any more  because I'll try another trick : */
/* comparison of not + typ_mvt_lea                                */
//#define  DEC_NOT_MAX_LEA  /* decrement the note when a taken    */
                          /* occures at leaf                      */

//#define  INC_POS_NOT      /* increment the note depending on the*/
                          /* the position of the piece on board   */
                          /* during the simulation                */
/* organization   should perhaps use a dynamic module */

//#define TEST           /*include code or module to test the pgm   */
//#define ALWAYS_STATS   /*alwways dsiplay statistics when playing  */ 
/******************************************************************/
#if BORLAND_C == 1
#include 	<sys/timeb.h>
#if DOS == 1
#include <bios.h>
#endif
#else
#include 	<sys/time.h>
#endif

#include 	<time.h>
 
/* #define BORLAND_C   0 */			/* 0 = Non 1 = OUI */
/*#define GCC	    1 */
 

#if BORLAND_C == 1
#if DOS == 1
#define ESC 27
#endif
#endif

#if LINUX  == 1
const struct sigaction st_sig_act;	/* gestion message system */ 
#endif


/* directive conditionelle por debug */
#define CPT_ITE 0		/* 1 pour inclusion du code 0 sinon */
#define CTL_DEP_ECH_PIE 0	/*  pour controle dephasage entre echiquier et piece */ 



/* Longueur message utilisateur */
#define LEN_MES 200

/* Nombre de parametre a inclure dans messages */
#define NBR_MAX_PAR_MES 10
/* Longueur maximum du parametre */
#define LON_MAX_PAR_MES 80

/* Profondeur de hst_pro */
#define PRO_HST			300
/* Nombre maximum de mouvement initiaux */
#define MVT_INI			70

/* Profondeur maximum de simulation inferieur/superieur */
#define PRO_SIM_SUP_MAX 10
#define PRO_SIM_INF_MAX 20

/* number maximum of hash table to use*/  
#define NBR_MAX_HASH 10
/* Yes No  */
enum defv_bool 
{
	N=0,	
	Y=1
};
typedef enum defv_bool defv_bool;

/* type de rock */
#define PET_ROC			0
#define GRA_ROC 		1

/* Piece side  */
enum e_side 
{
	BLANC=0,	
	NOIR=1
};
typedef enum e_side  e_side;

/* Valeur possible pour  le camp */
/*
#define BLANC			0
#define NOIR    		1*/
#define VIDE                   -1
#define HOR_ECH                -2

/* Type de mouvements */
#define PAS_DE_MVT		0
/* cavalier tour fou reine  */
#define TFR_DEP			1
#define TFR_PRI			2
/* roi */
#define ROI_DEP			3
#define ROI_PRI		        4
#define ROI_GRA_ROC		5
#define ROI_PET_ROC		6
/* cavalier */
#define CAV_DEP			7
#define CAV_PRI			8
/* pion */
#define PIO_1C        		9
#define PIO_2C 	       		10
#define PIO_PRI			11
#define PIO_PRI_PAS		12

#define PIO_PRO_REI	        13
#define PIO_PRO_FOU	        14
#define PIO_PRO_TOU	        15
#define PIO_PRO_CAV	        16

#define PIO_PRO_REI_PRI	        17
#define PIO_PRO_FOU_PRI	        18
#define PIO_PRO_TOU_PRI	        19
#define PIO_PRO_CAV_PRI	        20

#define NBR_TYP_MVT             21

/* */
#define NOT_MIN        -32768    /* le plus grand nombre negatif pour le type int */
#define NOT_ROC_TOU      -10    /* empecher le roc  */

#define FOR_PIO           100
#define FOR_CAV           300
#define FOR_FOU           300
#define FOR_TOU           500
#define FOR_REI           900
 /* pour analyse couverture */
#define FOR_ROI             1

#define NOT_MAT           9000
 /* rapport de force limite au dessu duquel on arrete la descente (cut)*/  
#define RAP_FOR_LIM_CUT   700
/* increment or devrement the note when taking occure without gain */    
#define  INC_PRI_NUL       70     
/* perte limite au dessus delaquelle il faut essayer de negocier un pat */
#define  LIMIT_PAT       2000     
/* choice made with random when possible */
#define  RANDOM_CHOICE      1
/* how many among the best move are chose for random choise?  */
#define RANDOM_CHOICE_MAX  6

/* NOT_MAT_MIN est un seuil:
	au dessus on est certain que la note represente un mat et ce malgre le decrement de 10
	par profondeur (900 - 10 * 40 mvt successif )=500. La machine capable d'analyser sur une 
	prfondeur de 40 mvt n'existe pas encore.

	au dessous la note represente seulement des prises de piece car si toutes les piece etaient prises
	la note ne depasserai pas 400.

	La not d'une branche comportant un mat n'inclura plus les eventuelle prises de cette branche pour 
	pouvoir aller le plus rapidement au mat s'il est inevitable.
	
	On pourra savoir en combien de mvt successifs le mat inevitable pourra avoir lieu avec:
	Nbr mvt = (NOT_MAT - note)/10.
	cad si note = 880 mat inevitable en 2 coups.  
 
*/
#define NOT_MAT_MIN      5000   
/* at each depth the not mat is decremented by that value
   to ensure going as fast as possible to checkmat */
#define DEC_MAT           100
/* at leaf only a percentage of the note is taken because of the risk
   that another piece is taken  at next level without knowing */
#define LEA_PER_DEC       0.7


/* libelle colone <-> N de colonne */
#define A              1
#define B              2
#define C              3
#define D              4
#define E              5
#define F              6
#define G              7
#define H              8

/* libelle case <-> numero de case  */
#define A1              11
#define A2              21 
#define A3              31
#define A4              41
#define A5              51
#define A6              61
#define A7              71
#define A8              81

#define B1              12
#define B2              22
#define B3              32
#define B4              42
#define B5              52
#define B6              62
#define B7              72
#define B8              82

#define C1              13
#define C2              23
#define C3              33
#define C4              43
#define C5              53
#define C6              63
#define C7              73
#define C8              83

#define D1              14
#define D2              24
#define D3              34
#define D4              44
#define D5              54
#define D6              64
#define D7              74
#define D8              84

#define E1              15
#define E2              25
#define E3              35
#define E4              45
#define E5              55
#define E6              65
#define E7              75
#define E8              85

#define F1              16
#define F2              26
#define F3              36
#define F4              46
#define F5              56
#define F6              66
#define F7              76
#define F8              86

#define G1              17
#define G2              27
#define G3              37
#define G4              47
#define G5              57
#define G6              67
#define G7              77
#define G8              87

#define H1              18
#define H2              28
#define H3              38
#define H4              48
#define H5              58
#define H6              68
#define H7              78
#define H8              88

/* Modification de coordonnee roi, reine, fou, tour */
#define HH             +=10              /* haut        1 case */
#define HD             +=11              /* haut droite 1 case */
#define DD             +=1               /* droite      1 case */
#define BD             -=9               /* bas droite  1 case */
#define BB             -=10              /* bas         1 case */
#define BG             -=11              /* bas gauche  1 case */
#define GG             -=1               /* gauche      1 case */
#define HG             +=9               /* haut gauche 1 case */

#define _HH             +=10              /* haut        1 case */
#define _HD             +=11              /* haut droite 1 case */
#define _DD             +=1               /* droite      1 case */
#define _BD             -=9               /* bas droite  1 case */
#define _BB             -=10              /* bas         1 case */
#define _BG             -=11              /* bas gauche  1 case */
#define _GG             -=1               /* gauche      1 case */
#define _HG             +=9               /* haut gauche 1 case */


/* Modification de coordonnee cavalier */
#define CHD             +=21              /* cavalier haut droite */
#define CDH             +=12              /* cavalier droite haut */
#define CDB             -=8               /* cavalier droite bas  */
#define CBD             -=19              /* cavalier bas droite  */
#define CBG             -=21              /* cavalier bas gauche  */
#define CGB             -=12              /* cavalier gauche bas  */
#define CGH             +=8               /* cavalier gauche haut */
#define CHG             +=19              /* cavalier haut gauche */

/* Modification de coordonnee pion blanc */
#define PH1            +=10              /* pion haut 1        */
#define PH2            +=20              /* pion haut 2        */
#define PHG            +=9               /* pion haut gauche   */
#define PHD            +=11              /* pion haut droite   */

#define PB1            -=10              /* pion bas 1         */
#define PB2            -=20              /* pion bas 2         */
#define PBG            -=11              /* pion bas gauche    */
#define PBD            -=9               /* pion bas droite    */


/* Message d'erreur */
enum e_msg_nbr 
{
      MSG_CHA_MES_LON_MSG		,
      MSG_CHO_COT_INV			,
      MSG_CHO_INC_INV			,
      MSG_CHO_NIV_INC_INV		,
      MSG_CHO_NIV_INV			,
      MSG_EXI_MVT_INI_NON		,
      MSG_JOU_CTFR_ROI_ECH		,
      MSG_JOU_GRD_ROC_ROC_IMP		,
      MSG_JOU_INT_COL_ARR_INV		,
      MSG_JOU_INT_COL_DEP_INV		,
      MSG_JOU_INT_LIG_ARR_INV		,
      MSG_JOU_INT_LIG_DEP_INV		,
      MSG_JOU_PAS_DE_PIE		,
      MSG_JOU_PAS_VOT_TOU		,
      MSG_JOU_PET_ROC_ROC_IMP		,
      MSG_JOU_PIB_2CA_IMP		,
      MSG_JOU_PIB_ARR_OCC		,
      MSG_JOU_PIB_INT_OCC		,
      MSG_JOU_PIB_MVT_IMP		,
      MSG_JOU_PIB_PAS_PIE_PRI		,
      MSG_JOU_PIN_2CA_IMP		,
      MSG_JOU_PIN_MVT_IMP		,
      MSG_JOU_PIO_PIE_PRO_INV		,
      MSG_JOU_PIO_ROI_ECH		,
      MSG_JOU_POS_REP_P3F		,
      MSG_JOU_PRI_PRO_CAM		,
      MSG_JOU_ROI_GRD_ROC_IMP		,
      MSG_JOU_ROI_PET_ROC_IMP		,
      MSG_JOU_ROI_ROI_ECH		,
      MSG_MVT_CAV_MVT_IMP		,
      MSG_MVT_FOU_MVT_IMP		,
      MSG_MVT_FOU_SAU_PIE		,
      MSG_MVT_REI_MVT_IMP		,
      MSG_MVT_REI_SAU_PIE		,
      MSG_MVT_ROI_MVT_IMP		,
      MSG_MVT_TOU_MVT_IMP		,
      MSG_MVT_TOU_SAU_PIE		,
      MSG_PAS_COU_PRE			,
      MSG_RST_DSK_ERR_DON		,
      MSG_RST_DSK_ERR_FIC		,
      MSG_SAV_DSK_ERR_FIC		,
      MSG_SIMU_HST_SIZ			,
      MSG_SIMU_MAT_BLANC		,
      MSG_SIMU_MAT_NOIR			,
      MSG_SIMU_PAT_BLANC		,
      MSG_SIMU_PAT_NOIR			,
      MSG_TYP_LIB_CAV			,
      MSG_TYP_LIB_REI			,
      MSG_VER_AFF_NOT_IMP		,
      MSG_VER_CHO_PRO_PIO		,
      MSG_LIB_BLANC                     ,
      MSG_LIB_NOIR                      ,
      MSG_NBR  	                        
};

#define TRUE  1
#define FALSE 0

#define INI_MVT_INI_YES 1
#define INI_MVT_INI_NO  0

#define INI_RAP_FOR_YES 1
#define INI_RAP_FOR_NO  0

#define USE_HASH_YES 1
#define USE_HASH_NO  0


/************************************************************/
/* variable test */
# if CPT_ITE == 1
unsigned long int cpt;
# endif
/* tableau correspondance posx <-> libelle colonne */
static const char tc_lib_cas[100] [3]=
{
  "  ",/*xx*/ "  ","  ","  ","  ","  ","  ","  ","  ", /*00*/"  ",
  "  ",/*01*/ "A1","B1","C1","D1","E1","F1","G1","H1", /*01*/"  ",
  "  ",/*02*/ "A2","B2","C2","D2","E2","F2","G2","H2", /*02*/"  ",
  "  ",/*03*/ "A3","B3","C3","D3","E3","F3","G3","H3", /*03*/"  ",
  "  ",/*04*/ "A4","B4","C4","D4","E4","F4","G4","H4", /*04*/"  ",
  "  ",/*05*/ "A5","B5","C5","D5","E5","F5","G5","H5", /*05*/"  ",
  "  ",/*06*/ "A6","B6","C6","D6","E6","F6","G6","H6", /*06*/"  ",
  "  ",/*07*/ "A7","B7","C7","D7","E7","F7","G7","H7", /*07*/"  ",
  "  ",/*08*/ "A8","B8","C8","D8","E8","F8","G8","H8", /*08*/"  ",
  "  ",/*00*/ "  ","  ","  ","  ","  ","  ","  ","  ", /*00*/"  "
};


/**********************************************************************/
/*  structure enum type def for global data                           */
/**********************************************************************/

/* fonction mouvement */
typedef void (*tpf_xxx)(void);
typedef void (*def_tpf_res_pie) (void);
typedef int (*def_tpf_cas_ctl[19])     (int n,int i_cam_adv);
typedef const int def_ti_ech_dir_rei[100][100];
typedef const int def_ti_ech_dir_tou[100][100];
typedef const int def_ti_ech_dir_fou[100][100];
typedef const int def_ti_ech_dir_cav[100][100];
typedef const int def_ti_ech_dir_roi[100][100];
typedef const int def_ti_ech_dir_pio_noi[100][100];
typedef const int def_ti_ech_dir_pio_bla[100][100];
typedef const int def_ti_ech_dir_inc[100][100];
/* Mark  each  position of pieces depending on situation                          */
typedef  const char def_not_pos;
typedef def_not_pos tdef_not_pos[100];
typedef def_not_pos * tptdef_not_pos[3];
typedef def_not_pos ** ptptdef_not_pos;

int i_cot_aff;
int i_side;
int i_first_time_init;
/*   tableau des pièces un par camp  */
/* opti: voir si un seul tableau possible */
struct st_pie
{
  int i_Id;				/* Id de la piece */	
  int i_pie_sui;		        /* pièce suivante */
  int i_pie_pre;                        /* pièce precedente  */
  int i_pos_n;                          /* position échiquier  */
  int i_for; 			        /* force  */
  int i_big_inc_ene;                    /* force incrementé pour eviter sacrifice inutile de piece */
  int i_big_inc_pro;                    /* force incrementé pour eviter sacrifice inutile de piece */
  int i_lit_inc_ene;                    /* force incrementé pour eviter sacrifice inutile de piece */
  int i_lit_inc_pro;                    /* force incrementé pour eviter sacrifice inutile de piece */
  char c_lib_pie;			/* Libelle piece  */
  char c_typ_pie;			/* Type  piece  */
  void (*pf_ini_hst_mvt) (void);	/* ptr sur fonction in_mvt_XXX   */
  int (*pf_jou)  (char c_rem_pio);	/* ptr sur fonction joue pièce	 */      
  int (*pf_ech_par) (int n,int nr);     /* ptr sur fonction echec par  */
  int (*pf_ctl_par) (int n,int nr);     /* ptr sur fonction cas_tl_xxx  */
  defv_bool defv_mvt_uni_dir ;    	/* mvt unique pour direction   */
  defv_bool defv_typ_mvt_tou ;          /* type de mvt selon colone oui = 1 pour reine et tour */   

  defv_bool defv_typ_mvt_fou ;          /* type de mvt selon oblique oui= 1 pour reine et fou */

#ifdef INC_POS_NOT
  def_inc_not  *def_ptc_inc_not;        /* pointer on the table used to increment the mark (note) */
                                        /* this pointer is loaded in pro_sui                      */
#endif
  ptptdef_not_pos  ptptdef_not_pos_pie; /* table containing pointer on table not_pos */

};

/* tableau echiquier accès par l'emplacement  */
struct st_ech
{
  int i_cam;
  /* opti perf: pourrait peut etre utiliser une adresse sur piece */ 
  int i_Id;
};



/* Table des messages utilisateur */
/*char tc_msg_txt[NBR_MSG][LEN_MES];*/
typedef char *def_tptc_msg[MSG_NBR];

/* strucuture parametre messages */
struct st_par
{
  int i_nbr_par;                /* nombre de parametres */
  char tc_par_mes [NBR_MAX_PAR_MES][LON_MAX_PAR_MES];
};

enum {e_pas_aff,e_aff_not_1,e_aff_not_2,st_aff_dim} e_cur_aff;		/* affichage en cours */

/* used be able to print result in several page  with teletype mode */
struct st_aff
{
  int i_nbr_lig_pag;
  int i_cur_pag;
  int i_fir_pag;
  int i_las_pag;
  int (*pf_aff)();
};

/* structure used to register the allocated memmory (as an index) */ 
struct st_ind
{
  void * pv_mem;               /* pointer on the memory registered*/
  struct st_ind *pst_pre_ind;  /* link to previous index*/
} ;

#ifdef UPLOAD_BRANCHE  
/*  */

struct  st_bra_nod
{
  char i_pos_n;                 	/* ariving square*/
  char i_typ_mvt;		        /* moe type  */
  char i_pos_ori_n;                     /* departure square */
  struct st_bra_nod *pst_nxt_nod;       /* next _subsequent node (move)*/
};

struct st_branche
{  

  long  li_ini_nod_qua;                   /* initial node quantity  */
  long  li_tot_nod_nbr;                   /* total number of nodes in all blocks */
  int   i_inc_per;                        /* increment percentage when a block i full */

  long  li_byt_siz;                       /* total size in byte for all blocks*/
  int   i_tot_inc_nbr;                    /* total number of increment used */

  struct st_bra_nod * pst_blo_beg;        /* beginnning of memory block */
  struct st_bra_nod * pst_blo_end;        /* end of  memory block */
  struct st_bra_nod * pst_fre_new_nod;    /* next free node to use  */

  struct st_bra_nod * pst_fre_old_nod;    /* head of the garbage collector */
  struct st_bra_nod * pst_las_old_nod;    /* tail of the garbage collector */

  struct st_bra_nod * pst_nxt_fre_nod;    /* next free node to use  */

  struct st_ind * pst_ind;                /* memory block liste   */

};

#endif
/* the data used to construct maintain  and acces the hash  */
struct st_hash {

  struct st_arr * pst_arr_beg;      /* beginnning of memory for arriving field */
  struct st_arr * pst_arr_end;      /* end  memory for arriving field */
  struct st_arr * pst_arr_fre;      /* next free bloc to use  */

  struct st_dep * pst_dep_cur_l1;   /* current movment at level one */
  struct st_dep * pst_dep_cur_l2;   /* current movment at level two */
  struct st_dep * pst_dep_cur_l3;   /* current movment at level three */

  struct st_dep * pst_dep_beg;      /* beginnning of memory for departure field */
  struct st_dep * pst_dep_end;      /* beginnning of memory for departure field */
  struct st_dep * pst_dep_fre;      /* next free bloc to use */
  struct st_ind * pst_ind;          /* memory block liste   */

  int   i_mvt_qua_ave;              /* initial node quantity at the top of the tree */
                                    /* its the number of initial move             */
                                    /* used as a base to compute the total node quantity*/
                                    /* for each blocs dep and arr                       */
  int   i_inc_per;                  /* increment percentage when a bloc i full */
  long  li_mvt_qua_tot;              /* movement quantity average for ech level  soon deprecated */
  long  li_dep_mov_qua;              /* inial node quantity use for the bloc dep  */
  long  li_arr_mov_qua;              /* inial node quantity use for the bloc arr  */

  long  l_dep_siz;                  /* size in byte of bloc depart */
  int   i_dep_inc_nbr;              /* number of increment in bloc depart */
  long  l_dep_seg_nbr;              /* number of segment in bloc depart */

  long  l_arr_siz;                  /* size in byter of bloc arriving*/
  int   i_arr_inc_nbr;              /* number of increment in bloc arriving */
  long  l_arr_seg_nbr;              /* number of segment in bloc arriving */

  long  l_nbr_bra_fou;              /*number of banche found in the hash */
                                    /*its also the number of move at level 3 */
  long  l_nbr_bra_sek;              /*number of move seeked  in the hash */

};

/* st_dep f  */
struct st_dep
{ 
  int i_dep;                        /* Departure field */
  int i_typ_mvt;                    /* Type of move */
  struct st_arr  * pst_arr_nxt_lev; /* pointer on the movement in next*/
                                    /* depth-level or the datas colected  */
  struct st_dep * pst_nxt_dep;      /* pointer on the next movement having */
                                    /*the same arriving  field at same depth-level */
  int i_not;
  int i_typ_pri_lea;                /* if thereis a taken on the leaf node */
                                    /* uploaded with the best note */
#ifdef UPLOAD_BRANCHE  
  struct st_bra_nod *pst_not_max_bra_hea;  /* first move of the list for the not_max branche*/ 
  struct st_bra_nod *pst_not_max_bra_las;  /* last  move of the list for the not_max branche*/ 
#endif

};


/* tableau historique des coups */
struct st_hst_pro
{
  int i_pro;			        /* profondeur pour test du ptr pts_hst_pro 0= debut de partie */
  int i_dec_mat;		        /* décrement pour la note mat */
  defv_bool defv_exp_par;               /* rester au même niveau si deplacement suit une prise */
  const  tpf_xxx *ptpf_mvt;	        /* ptr sur fonction  mouvement */
  int i_cam;    	 	        /* camp	*/
  struct st_pie *pts_pie_pc;            /* pointeur sur piece propre camp  */        
  defv_bool defv_ene;                   /* Yes if adversary */
  int i_cam_adv;	         	/* camp	 adverse */
  struct st_pie *pts_pie_ca;            /* pointeur sur piece camp adverse */
  int i_Id;                 	        /* Idetifiant pièce */
  /* opti : voir si stockage de l'adresse serait pas plus performant */
  int i_pos_n;                 	        /* pos. échiquier*/
  int i_typ_mvt;		        /* type mouvement */
  defv_bool defv_dir_sui;		        /* direction suivante*/
  int i_pos_ori_n;                      /* pos. origine	*/
  int i_Id_2p;                          /* Id deuxième pièce */
  int i_pos_n_R_p_c;	                /* Position n roi propre camp	*/
  int i_pos_n_R_c_a;	                /* Position n roi camp	adverse	*/
  defv_bool defv_ech ;		        /* echec par pièce principale 
                                           ou secondaireou par demasquge */
  int i_ech_dir_pri;	                /* dir. pièce principale ou secondaire
                                            causant echec sur camp adverse
				           (si valeur  <> 0 alors echec) */
  int i_ech_dir_dem;	                /* dir.demasaquage causant eventuel
				           echec sur camp  adverse  */
  int i_ech_dir_dem_p_c;                /* direction demasaquage causant echec	
				           sur propre camp  */
  defv_bool defv_pat ;    	         	/* pat   */
  defv_bool defv_mat ;	        	/* mat 	*/
  /*  int ti_not[2];  */                /* note par camp */
  int i_not;		        	/* note */
  int i_typ_mvt_pri_lea;	        /* type of move at the leaf */
  int i_not_max;		        /* note maximum */
  int i_not_max_typ_mvt_pri_lea;        /* type of move at the leaf for not max */
  int i_rap_for;                        /* rapport de force à zero en debut de simu */
  int ti_roc_cam_typ[2][2];             /* [cam][type] type: 0= petit 1= grd 
                                           Si = 1 alors piece deplacé
                                           bug Suppression non signalé*/
  unsigned long int uli_cpt_cou;        /* Compteur de coup */
  unsigned long int uli_cpt_mat;        /* Compteur de mat */
  unsigned long int uli_cpt_pat;        /* Compteur de pat */
  unsigned long int uli_cpt_mat_rf;     /* Compteur de mat rapport de force = 0 */
  unsigned long int uli_cpt_pat_rf;     /* Compteur de pat rapport de force = 0 */
  unsigned long int uli_nbr_fou;        /* Compteur de fouchettes */
  int i_nbr_pri_pie;                    /* Compteur de prises faites par une piece 
				           raz à chaque chgt de piece */
  /* variable test */
# if CPT_ITE == 1
  unsigned long int cpt;
# endif
#ifdef STHASH1  

  int i_sto_res_hash_nbr;                /* hash number to use with sto_res()*/
  defv_bool defv_store_result;           /* do not store if FALSE */

                                         /* do not call if < 0 */
  int i_mem_hash_nbr;                    /* indexe used to access the table of struct st_hash*/
                                         /* used as argument with fre_mem() and ini_mem()*/
                                         /* do not call if < 0 */

  int i_nbr_ele_tpf_hash;                /* number of element in *tpf_hash*/
  int ti_arg_fct_hash_add[NBR_MAX_HASH]; /* argument to pass to fct add hash */
  int (*tpf_hash_add [NBR_MAX_HASH])     /* list of functions to call to maintain */ 
        (struct st_hash *tpst_hash);     /* all the hash table */ 
  
#endif
#ifdef STHASH  
        			    
    int (*pf_hash_add_lx)(struct st_hash *pst_hash1); /* deprecated */ 
#endif
#ifdef UPLOAD_BRANCHE  
  struct st_bra_nod *pst_not_max_bra_hea;  /* first move of the list for the not_max branche*/ 
  struct st_bra_nod *pst_not_max_bra_las;  /* last  move of the list for the not_max branche*/ 
#endif

};
/* Type de note pour mise en tableau */
enum typ_not
{
  nbr_mvt,
  nbr_mat,
  nbr_mat_inc,
  nbr_pat,
  nbr_pat_inc,
  nbr_mat_rf,
  nbr_pat_rf,
  nbr_fou,
  mob_pie,
  cou_pie,
  pos_pie,
  NBR_TYP_NOT 
};

/* tableau note du mvt initial */
#define ST_NOT_MVT_INI   struct	 st_not_mvt_ini  
struct st_not_mvt_ini
{
  int 	i_num_mvt;	/* Numero du mvt */
  int   i_pro_sim;		/* profondeur de simulation */	 
  int	i_inc_pro_sim;		/* incrément profondeur de simulation*/
  int 	i_cam;	  	/* cam propre */
  int 	i_cam_adv;	/* cam adverse*/
  int 	i_Id;		/* pièce mouvemente*/
  char 	c_lib_pie;      /*  libelle piece*/
  int 	i_typ_mvt;	/* type mouvement */
  int 	i_typ_mvt_pri;	/* type mouvement prise */
  int 	i_pos_ori_n;    /* pos d'origine échiquier */
  int 	i_pos_arr_n;    /* pos arrivee échiquier*/
  int 	i_nbr_rep_sit;  /* Nombre de répétition situation */
  int   i_not_pri;	/* Note prise */
  int   i_typ_mvt_pri_lea; /* A taken is done on the leaf node */

  
  /* struct st_hst_pro_sta 	*/	/* statistique par profondeur */
  /*tst_pro_sta[PRO_SIM_INF_MAX]; */	/* marge de 1 poste car profondeur de depart */

  struct st_not
  {
    float f_not_cam_bru[2];  /* donnée cumulée brute(cad non pondérée) */
    float f_not_cam_pon[2];  /* note pondrée par camp  */
    float f_not_dif_pon;     /* difference pondérée pc-ca pond */
    int   i_not_pon_rec;     /* fifférence pondérée et recadrée  */
  } tst_not[NBR_TYP_NOT];

  /* Note globale */
  float f_tot_dif_pon;		/* total pondéreé de la note non recadrée */
  float	f_tot_rec_pon;       	/* total pondéré de la note recadrée */ 
#ifdef UPLOAD_BRANCHE  
/*  */
  struct st_bra_nod *pst_bra_nod;
#endif

};
/* structure pour evaluation rapport de force */
struct st_eva_sit
{
  int    ti_tot_for[2];        /* Totalisation force des piece par camp */
  int    i_not_for;            /* Force pc - ca */
  float  tf_cou[2];            /* Evaluation couverture par camp avant mouvement */
  float  f_cou;                /* cou pc - cou ca */
  float  tf_mob[2];             /* mobilité par camp */
  float  f_not_mob;              /* mob pc- mob ca */
  /* Pondération ou ordre de tri */
  int ti_coe_pon[NBR_TYP_NOT];

  /* maximum /minimu rencontrée ds la selction des meilleurs mvts pour cal note recadrée */

  struct st_int_not
  {
    float max;
    float min;
  } tst_int_not[NBR_TYP_NOT];


};

/* Sous structure de not_mvt_ini pour evaluation mat/pat par profondeur */

struct st_hst_pro_sta	
{
  int i_cam;				/* Camp jouant a cette  pro	*/
  int i_inc;				/* Si increment profondeur = 1	*/
  float f_dat_pro [NBR_TYP_NOT];	/* Data par profondeur  non pondéré */
};
typedef struct  st_hst_pro_sta  def_tst_pro_sta [MVT_INI][PRO_SIM_INF_MAX]; 

/* indexe pour retrier les mouvement */
struct st_tri_mvt_ini
{
  ST_NOT_MVT_INI *pts_tri_rec;

};

/* Memorisation de position des pieces detecter les cas de partie nulle */
struct st_pos
{
  int i_pos_n [2][17];		/* position cam Id  */
  int ti_roc_cam_typ[2][2];     /* [cam][type] type: 0= petit 1= grd */
  int i_cam_jou;		/* Camp qui doit jouer */
  /* info pour éviter de tester l'ensemble des pieces */
  int i_nbr_pie[2];		/* Nombre de piece par camp */
  int i_som_pos[2];		/* Somme des positions */
  int i_nbr_pie_tot;		/* Nombre de pieces total */
  int i_som_pos_tot;		/* Somme des position */
};


typedef void (*def_pf_not_cal)(void);
struct st_ver
{
  int (*pf_eva) (void);
  int i_pro_sim_niv;
  int i_pro_sim_inc;
  char * ptc_lib;
  int i_ran_cho_max;
};

enum e_ver
{ 
  V00,
  V01,
  V02,
  LOW1,
  MEDIUM1,
  HIGH1,
  VER_NBR
};
/* supposed to become  the global acces key for module evaluation */
struct st_evaluation
{
  int i_number_of_initial_move;
  //ST_NOT_MVT_INI  *ts_not_mvt_ini;
  //  def_tst_pro_sta tst_pro_sta;
  //struct  st_hst_pro_sta tst_pro_sta_[MVT_INI][PRO_SIM_INF_MAX]; 
  //struct  st_hst_pro_sta *tst_pro_sta_[MVT_INI]; 

  int i_best_move_nbr;
};
struct st_evaluation  *pst_evaluation;
struct st_evaluation  st_evaluation;


/* to avoid multiple inclusion of this entire file */
#endif
